unit DKCReallocBenchmarkUnit;

interface

uses Windows, BenchmarkClassUnit, Classes, Math;

type

  TDKCReallocThreads = class(TFastcodeMMBenchmark)
  public
    procedure RunBenchmark; override;
    class function GetBenchmarkName: string; override;
    class function GetBenchmarkDescription: string; override;
    class function GetSpeedWeight: Double; override;
    class function GetCategory: TBenchmarkCategory; override;
  end;

implementation

uses SysUtils;

type

  TDKCReallocThread = class(TThread)
     FBenchmark: TFastcodeMMBenchmark;
     procedure Execute; override;
  end;

procedure TDKCReallocThread.Execute;
const
 LoopCount = 60 * 1000;
var
 I: Integer;
 S: string;

begin
 RandSeed := 100;
 for I := 1 to LoopCount do
  begin
   SetLength(S, Random(16*LoopCount));
   FBenchmark.UpdateUsageStatistics;
  end;
end;

class function TDKCReallocThreads.GetBenchmarkDescription: string;
begin
  Result := 'A benchmark that tests a special case of realloc. '
          + 'Uses code from the QC report 24077. '
          + 'Benchmark submitted by Dennis Kjaer Christensen.';
end;

class function TDKCReallocThreads.GetBenchmarkName: string;
begin
  Result := 'DKCRealloc';
end;

class function TDKCReallocThreads.GetCategory: TBenchmarkCategory;
begin
  Result := bmMultiThreadRealloc;
end;

class function TDKCReallocThreads.GetSpeedWeight: Double;
begin
  Result := 0.5;
end;

procedure TDKCReallocThreads.RunBenchmark;
var
 DKCReallocThread : TDKCReallocThread;

begin
  inherited;
  DKCReallocThread := TDKCReallocThread.Create(True);
  DKCReallocThread.FreeOnTerminate := False;
  DKCReallocThread.FBenchmark := Self;
  DKCReallocThread.Resume;
  DKCReallocThread.WaitFor;
  DKCReallocThread.Free;
end;

end.